@@ -34,7 +34,7 @@ import zipfile
 from common import gajim
 from plugins import GajimPlugin
 from plugins.helpers import log_calls, log
-from conversation_textview import ConversationTextview
+from htmltextview import HtmlTextView
 from dialogs import WarningDialog, HigDialog, YesNoDialog
 from plugins.gui import GajimPluginConfigDialog
 
@@ -61,7 +61,6 @@ class PluginInstaller(GajimPlugin):
 
     @log_calls('PluginInstallerPlugin')
     def init(self):
-        self.description = _('Install and upgrade plugins from ftp')
         self.config_dialog = PluginInstallerPluginConfigDialog(self)
         self.config_default_values = {'ftp_server': ('ftp.gajim.org', ''),
                                       'check_update': (True, ''),
@@ -80,7 +79,7 @@ class PluginInstaller(GajimPlugin):
         elif gajim.version.startswith('0.16.10'):
             self.server_folder = 'plugins_gtk3'
         else:
-            self.server_folder = 'plugins_0.16'
+            self.server_folder = 'plugins_0.16_zip'
 
     @log_calls('PluginInstallerPlugin')
     def activate(self):
@@ -257,9 +256,9 @@ class PluginInstaller(GajimPlugin):
 
         self._clear_available_plugin_info()
 
-        self.plugin_description_textview = ConversationTextview(None)
+        self.plugin_description_textview = HtmlTextView()
         sw = self.xml.get_object('scrolledwindow1')
-        sw.add(self.plugin_description_textview.tv)
+        sw.add(self.plugin_description_textview)
 
         self.xml.connect_signals(self)
         self.window.show_all()
@@ -358,7 +357,10 @@ class PluginInstaller(GajimPlugin):
                 plugin.activatable = False
             max_row = [plugin, plugin.name, is_active, plugin.activatable, icon]
             # support old plugin system
-            row_len = len(self.installed_plugins_model[0])
+            if len(self.installed_plugins_model):
+                row_len = len(self.installed_plugins_model[0])
+            else:
+                row_len = 5
             row = max_row[0: row_len]
             self.installed_plugins_model.append(row)
 
@@ -367,9 +369,9 @@ class PluginInstaller(GajimPlugin):
     def available_plugins_treeview_selection_changed(self, treeview_selection):
         model, iter = treeview_selection.get_selected()
         self.xml.get_object('scrolledwindow1').get_children()[0].destroy()
-        self.plugin_description_textview = ConversationTextview(None)
+        self.plugin_description_textview = HtmlTextView()
         sw = self.xml.get_object('scrolledwindow1')
-        sw.add(self.plugin_description_textview.tv)
+        sw.add(self.plugin_description_textview)
         sw.show_all()
         if iter:
             self.plugin_name_label1.set_text(model.get_value(iter, C_NAME))
@@ -386,19 +388,9 @@ class PluginInstaller(GajimPlugin):
                 desc = '<body  xmlns=\'http://www.w3.org/1999/xhtml\'>' + \
                     desc + ' </body>'
                 desc = desc.replace('\n', '<br/>')
-            import inspect
-            args = inspect.getargspec(
-                self.plugin_description_textview.tv.display_html)
-            gajim_version = list(gajim.version.split('-', 1)[0].split('.'))
-            if (gajim_version >= [0, 16]):
-                # Gajim_0.16
-                self.plugin_description_textview.tv.display_html(desc,
-                    self.plugin_description_textview.tv,
-                    self.plugin_description_textview)
-            else:
-                self.plugin_description_textview.tv.display_html(desc,
-                    self.plugin_description_textview)
-            self.plugin_description_textview.tv.set_property('sensitive', True)
+            self.plugin_description_textview.display_html(desc,
+                self.plugin_description_textview, None)
+            self.plugin_description_textview.set_property('sensitive', True)
         else:
             self._clear_available_plugin_info()
 
@@ -479,7 +471,8 @@ class PluginInstaller(GajimPlugin):
             selection = self.available_treeview.get_selection()
             if selection.count_selected_rows() == 0:
                 root_iter = self.available_plugins_model.get_iter_root()
-                selection.select_iter(root_iter)
+                if root_iter:
+                    selection.select_iter(root_iter)
         scr_win = self.xml.get_object('scrolledwindow2')
         vadjustment = scr_win.get_vadjustment()
         if vadjustment:
@@ -523,8 +516,7 @@ class Ftp(threading.Thread):
             if not self.remote_dirs:
                 gobject.idle_add(self.progressbar.set_text,
                     _('Scan files on the server'))
-                self.ftp.retrbinary('RETR manifests_images.zip',
-                    self.handleDownload)
+                self.ftp.retrbinary('RETR manifests_images.zip', self.handleDownload)
                 zip_file = zipfile.ZipFile(self.buffer_)
                 manifest_list = zip_file.namelist()
                 progress_step = 1.0 / len(manifest_list)
@@ -596,29 +588,7 @@ class Ftp(threading.Thread):
         self.pulse = gobject.timeout_add(150, self.progressbar_pulse)
         gobject.idle_add(self.progressbar.set_text, _('Creating a list of files'))
         for remote_dir in self.remote_dirs:
-
-            def nlstr(dir_, subdir=None):
-                if subdir:
-                    dir_ = dir_ + '/' + subdir
-                list_ = self.ftp.nlst(dir_)
-                for i in list_:
-                    name = i.split('/')[-1]
-                    if '.' not in name:
-                        try:
-                            if i == self.ftp.nlst(i)[0]:
-                                files.append(i[1:])
-                                del dirs[i[1:]]
-                        except Exception, e:
-                            # empty dir or file
-                            continue
-                        dirs.append(i[1:])
-                        subdirs = name
-                        nlstr(dir_, subdirs)
-                    else:
-                        files.append(i[1:])
-            dirs, files = [], []
-            nlstr('/%s/%s' % (self.plugin.server_folder, remote_dir))
-
+            filename = remote_dir + '.zip'
             base_dir, user_dir = gajim.PLUGINS_DIRS
             if not os.path.isdir(user_dir):
                 os.mkdir(user_dir)
@@ -627,26 +597,17 @@ class Ftp(threading.Thread):
                 os.mkdir(local_dir)
             local_dir = os.path.split(user_dir)[0]
 
-            # creating dirs
-            for dir_ in dirs:
-                dir_ = dir_.replace(self.plugin.server_folder, 'plugins')
-                if os.path.exists(os.path.join(local_dir, dir_)):
-                    continue
-                os.mkdir(os.path.join(local_dir, dir_))
+            # downloading zip file
+            gobject.idle_add(self.progressbar.set_text,
+                _('Downloading "%s"') % filename)
+            self.buffer_ = io.BytesIO()
+            try:
+                self.ftp.retrbinary('RETR %s' % filename, self.handleDownload)
+            except ftplib.error_perm:
+                print 'ERROR: cannot read file "%s"' % filename
+            with zipfile.ZipFile(self.buffer_) as zip_file:
+                zip_file.extractall(os.path.join(local_dir, 'plugins'))
 
-            # downloading files
-            for filename in files:
-                gobject.idle_add(self.progressbar.set_text,
-                    _('Downloading "%s"') % filename)
-                full_filename = os.path.join(local_dir, filename.replace(
-                    self.plugin.server_folder, 'plugins'))
-                try:
-                    self.ftp.retrbinary('RETR /%s' % filename,
-                        open(full_filename, 'wb').write)
-                    #full_filename.close()
-                except ftplib.error_perm:
-                    print 'ERROR: cannot read file "%s"' % filename
-                    os.unlink(filename)
         self.ftp.quit()
         gobject.idle_add(self.window.emit, 'plugin_downloaded',
             self.remote_dirs)
