@@ -33,9 +33,9 @@ options = {
    'build_exe': {
        'includes': ['gtk.keysyms', 'dumbdbm', 'dbhash', 'bsddb', 'new',
             'goocanvas', 'Crypto.PublicKey.DSA', 'Crypto.Hash.HMAC',
-            'numbers', 'win32com.client', 'win32com.server', 'HTMLParser'],
+            'numbers', 'win32com.client', 'win32com.server', 'HTMLParser', 'pkg_resources'],
        'base': 'Win32GUI',
-       'packages': ['cffi', 'cryptography'],
+       'packages': ['cffi', 'cryptography', 'PIL', 'axolotl', 'google'],
 	   'bin_excludes': [
             'iconv.dll', 'intl.dll', 'libatk-1.0-0.dll',
             'libgdk_pixbuf-2.0-0.dll', 'libgdk-win32-2.0-0.dll',
@@ -57,7 +57,7 @@ options = {
 
 setup(
     name='Gajim',
-    version='0.16.5',
+    version='0.16.6',
     description='A full featured Jabber client',
     author='Gajim Development Team',
     url='http://gajim.org/',
