@@ -36,6 +36,7 @@ import os
 import subprocess
 import urllib
 import urllib2
+from urlparse import urlparse
 import webbrowser
 import errno
 import select
@@ -45,6 +46,7 @@ import shlex
 import caps_cache
 import socket
 import time
+import datetime
 
 from encodings.punycode import punycode_encode
 from string import Template
@@ -121,7 +123,10 @@ def idn_to_ascii(host):
     labels = idna.dots.split(host)
     converted_labels = []
     for label in labels:
-        converted_labels.append(idna.ToASCII(label))
+        if label:
+            converted_labels.append(idna.ToASCII(label))
+        else:
+            converted_labels.append('')
     return ".".join(converted_labels)
 
 def ascii_to_idn(host):
@@ -136,6 +141,14 @@ def ascii_to_idn(host):
         converted_labels.append(idna.ToUnicode(label))
     return ".".join(converted_labels)
 
+def puny_encode_url(url):
+    _url = url
+    if '//' not in _url:
+        _url = '//' + _url
+    o = urlparse(_url)
+    p_loc = idn_to_ascii(o.netloc)
+    return url.replace(o.netloc, p_loc)
+
 def parse_resource(resource):
     """
     Perform stringprep on resource and return it
@@ -616,12 +629,26 @@ def datetime_tuple(timestamp):
             - XEP-082 datetime strings have all '-' cahrs removed to meet
               the above format.
     """
-    timestamp = timestamp.split('.')[0]
-    timestamp = timestamp.replace('-', '')
-    timestamp = timestamp.replace('z', '')
-    timestamp = timestamp.replace('Z', '')
-    from time import strptime
-    return strptime(timestamp, '%Y%m%dT%H:%M:%S')
+    date, tim = timestamp.split('T', 1)
+    date = date.replace('-', '')
+    tim = tim.replace('z', '')
+    tim = tim.replace('Z', '')
+    zone = None
+    if '+' in tim:
+        sign = -1
+        tim, zone = tim.split('+', 1)
+    if '-' in tim:
+        sign = 1
+        tim, zone = tim.split('-', 1)
+    tim = tim.split('.')[0]
+    tim = time.strptime(date + 'T' + tim, '%Y%m%dT%H:%M:%S')
+    if zone:
+        tim = datetime.datetime.fromtimestamp(time.mktime(t))
+        zone = strptime.time(zone, '%H:%M')
+        zone = datetime.timedelta(hours=zone.tm_hour, minutes=zone.tm_min)
+        tim += zone * sign
+        tim = tim.timetuple()
+    return tim
 
 # import gajim only when needed (after decode_string is defined) see #4764
 
@@ -919,13 +946,11 @@ distro_info = {
         'Aurox Linux': '/etc/aurox-release',
         'Conectiva Linux': '/etc/conectiva-release',
         'CRUX': '/usr/bin/crux',
-        'Debian GNU/Linux': '/etc/debian_release',
         'Debian GNU/Linux': '/etc/debian_version',
         'Fedora Linux': '/etc/fedora-release',
         'Gentoo Linux': '/etc/gentoo-release',
         'Linux from Scratch': '/etc/lfs-release',
         'Mandrake Linux': '/etc/mandrake-release',
-        'Slackware Linux': '/etc/slackware-release',
         'Slackware Linux': '/etc/slackware-version',
         'Solaris/Sparc': '/etc/release',
         'Source Mage': '/etc/sourcemage_version',
@@ -1449,9 +1474,11 @@ def _get_img_direct(attrs):
     """
     Download an image. This function should be launched in a separated thread.
     """
-    mem, alt = '', ''
-    # Wait maximum 5s for connection
-    socket.setdefaulttimeout(5)
+    mem, alt, max_size = '', '', 2*1024*1024
+    if 'max_size' in attrs:
+        max_size = attrs['max_size']
+    # Wait maximum 10s for connection
+    socket.setdefaulttimeout(10)
     try:
         req = urllib2.Request(attrs['src'])
         req.add_header('User-Agent', 'Gajim ' + gajim.version)
@@ -1461,13 +1488,13 @@ def _get_img_direct(attrs):
         pixbuf = None
         alt = attrs.get('alt', 'Broken image')
     else:
-        # Wait 0.5s between each byte
+        # Wait 2s between each byte
         try:
-            f.fp._sock.fp._sock.settimeout(0.5)
+            f.fp._sock.fp._sock.settimeout(2)
         except Exception:
             pass
-        # Max image size = 2 MB (to try to prevent DoS)
-        deadline = time.time() + 3
+        # On a slow internet connection with ~1000kbps you need ~10 seconds for 1 MB
+        deadline = time.time() + (10 * (max_size / 1048576))
         while True:
             if time.time() > deadline:
                 log.debug('Timeout loading image %s ' % attrs['src'])
@@ -1490,7 +1517,7 @@ def _get_img_direct(attrs):
                 mem += temp
             else:
                 break
-            if len(mem) > 2*1024*1024:
+            if len(mem) > max_size:
                 alt = attrs.get('alt', '')
                 if alt:
                     alt += '\n'
@@ -1505,15 +1532,19 @@ def _get_img_proxy(attrs, proxy):
     """
     if not gajim.HAVE_PYCURL:
         return '', _('PyCURL is not installed')
-    mem, alt = '', ''
+    mem, alt, max_size = '', '', 2*1024*1024
+    if 'max_size' in attrs:
+        max_size = attrs['max_size']
     try:
         b = StringIO()
         c = pycurl.Curl()
         c.setopt(pycurl.URL, attrs['src'].encode('utf-8'))
         c.setopt(pycurl.FOLLOWLOCATION, 1)
-        c.setopt(pycurl.CONNECTTIMEOUT, 5)
-        c.setopt(pycurl.TIMEOUT, 10)
-        c.setopt(pycurl.MAXFILESIZE, 2000000)
+        # Wait maximum 10s for connection
+        c.setopt(pycurl.CONNECTTIMEOUT, 10)
+        # On a slow internet connection with ~1000kbps you need ~10 seconds for 1 MB
+        c.setopt(pycurl.TIMEOUT, 10 * (max_size / 1048576))
+        c.setopt(pycurl.MAXFILESIZE, max_size)
         c.setopt(pycurl.WRITEFUNCTION, b.write)
         c.setopt(pycurl.USERAGENT, 'Gajim ' + gajim.version)
         # set proxy
