@@ -27,7 +27,6 @@
 ##
 
 import os
-import sys
 import locale
 import re
 from time import time
@@ -38,9 +37,12 @@ from common import caps_cache
 import sqlite3 as sqlite
 import logger
 
+import logging
+log = logging.getLogger('gajim.c.optparser')
+
 class OptionsParser:
     def __init__(self, filename):
-        self.__filename = filename
+        self.__filename = os.path.realpath(filename)
         self.old_values = {}    # values that are saved in the file and maybe
                                                         # no longer valid
 
@@ -63,14 +65,23 @@ class OptionsParser:
                 line = line.decode('utf-8')
             except UnicodeDecodeError:
                 line = line.decode(locale.getpreferredencoding())
-            optname, key, subname, value = regex.match(line).groups()
+            match = regex.match(line)
+            if match is None:
+                log.warn('Invalid configuration line, ignoring it: %s', line)
+                continue
+            optname, key, subname, value = match.groups()
             if key is None:
                 self.old_values[optname] = value
                 gajim.config.set(optname, value)
             else:
                 if (optname, key) not in seen:
+                    if optname in self.old_values:
+                        self.old_values[optname][key] = {}
+                    else:
+                        self.old_values[optname] = {key: {}}
                     gajim.config.add_per(optname, key)
                     seen.add((optname, key))
+                self.old_values[optname][key][subname] = value
                 gajim.config.set_per(optname, key, subname, value)
 
         old_version = gajim.config.get('version')
@@ -225,6 +236,10 @@ class OptionsParser:
             self.update_config_to_01401()
         if old < [0, 14, 90, 0] and new >= [0, 14, 90, 0]:
             self.update_config_to_014900()
+        if old < [0, 16, 0, 1] and new >= [0, 16, 0, 1]:
+            self.update_config_to_01601()
+        if old < [0, 16, 4, 1] and new >= [0, 16, 4, 1]:
+            self.update_config_to_01641()
 
         gajim.logger.init_vars()
         gajim.logger.attach_cache_database()
@@ -912,3 +927,21 @@ class OptionsParser:
             gajim.config.set('use_stun_server', False)
         if os.name == 'nt':
             gajim.config.set('autodetect_browser_mailer', True)
+
+    def update_config_to_01601(self):
+        if 'last_mam_id' in self.old_values:
+            last_mam_id = self.old_values['last_mam_id']
+            for account in gajim.config.get_per('accounts'):
+                gajim.config.set_per('accounts', account, 'last_mam_id',
+                    last_mam_id)
+        gajim.config.set('version', '0.16.0.1')
+
+    def update_config_to_01641(self):
+        for account in self.old_values['accounts'].keys():
+            connection_types = self.old_values['accounts'][account][
+            'connection_types'].split()
+            if 'plain' in connection_types and len(connection_types) > 1:
+                connection_types.remove('plain')
+            gajim.config.set_per('accounts', account, 'connection_types',
+                ' '.join(connection_types))
+        gajim.config.set('version', '0.16.4.1')
