@@ -96,8 +96,8 @@ class StanzaSession(object):
         for j in (self.jid, self.jid.getStripped()):
             for event in gajim.events.get_events(self.conn.name, j, types=types):
                 # the event wasn't in this session
-                if (event.type_ == 'chat' and event.parameters[8] != self) or \
-                (event.type_ == 'printed_chat' and event.parameters[2].session != \
+                if (event.type_ == 'chat' and event.session != self) or \
+                (event.type_ == 'printed_chat' and event.control.session != \
                 self):
                     continue
 
