@@ -47,9 +47,11 @@ from common import gajim
 from common import ged
 from common.zeroconf import client_zeroconf
 from common.zeroconf import zeroconf
-from connection_handlers_zeroconf import *
-from common.connection_handlers_events import *
-
+from connection_handlers_zeroconf import ConnectionHandlersZeroconf, STATUS_LIST
+from common.connection_handlers_events import ZeroconfPresenceReceivedEvent, \
+    RosterInfoEvent, OurShowEvent, MessageErrorEvent, ConnectionLostEvent, SignedInEvent, \
+    MessageSentEvent, RosterReceivedEvent, InformationEvent, ZeroconfNameConflictEvent
+import nbxmpp
 import locale
 
 class ConnectionZeroconf(CommonConnection, ConnectionHandlersZeroconf):
